--[[
  Premo Power version 2.0.1 release
  Unit Power movable bar for Paladins and Priests
  By Azmaedus 
  6.0.3 compatible
--]]

-- Lua APIs
local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring = string.format, tostring
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget

-- Load the Ace libraries --
PremoPower = LibStub("AceAddon-3.0"):NewAddon("PremoPower", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("PremoPower")

local LFX = LibStub("LibFrameX-1.0")
if not LFX then return end --fail

local LTC = LibStub("LibTColor-1.0")
if not LTC then return end --fail

-- ---------------------------------------------
-- PremoPower globals
-- ---------------------------------------------
PremoPower.gVersion = GetAddOnMetadata("PremoPower", "Version")
PremoPower.gVersionID = {2,0,0,1,1} -- release major, release minor, build major, build minor, type (alpha 0, beta 1, release 2)
PremoPower.gCurReleaseDate = "01/24/2015"
_G["PremoPower"] = PremoPower

--local currentSpecName = currentSpec and select(2, GetSpecializationInfo(currentSpec)) or "None"
--name, iconTexture, tier, column, rank, maxRank, isExceptional, meetsPrereq, previewRank, meetsPreviewPrereq = GetTalentInfo(tabIndex, talentIndex, inspect, pet, talentGroup)
--GetNumTalentTabs()

PremoPower.gPlayerInfo = {
  class = string.upper( select( 2, UnitClass( 'player' ) ) ),
  level = UnitLevel("player"),
  spec = GetSpecialization(),
}

PremoPower.gBackDrop = {
	bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
    edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
    tile = true, tileSize = 4, edgeSize = 4,
    insets = { left = 1, right = 1, top = 1, bottom = 1 }
}

local BackDropAlpha = 0.3

-- Class specific
local MAX_PALADIN_BLIP = 5
local MAX_PRIEST_DISC_BLIP = 5
local MAX_ROGUE_BLIP = 5

local SPEC_PRIEST_DISC = 2
local SPEC_PRIEST_SHADOW = 3

local moduleOptions = {}    	-- Table for [Load On Demand] module options registration

-- ---------------------------------------------
-- create minimap icon
-- ---------------------------------------------
function PremoPower:CreateMinimapIcon()
	local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
	local plugin = ldb:NewDataObject("PremoPower", {
		type = "data source",
		text = "0",
		icon = "Interface\\AddOns\\PremoPower\\media\\icon",
	})

	function plugin.OnClick(self, button)
		if button == "LeftButton" then
			PremoPower:ShowConfig()
		elseif button == "RightButton" then
			if PremoPower.db.profile.hideframes then
				LFX:setVisible(true)
				PremoPower:OnUpdateFrames()
				PremoPower.db.profile.hideframes = false
			else
				LFX:setVisible(false)
				PremoPower.db.profile.hideframes = true
			end
		end
	end

	local hint = L["ICON_CLICK"]
	local hint2 = L["ICON_RCLICK"]
	function plugin.OnTooltipShow(tt)
		tt:AddLine(L["ADDON_NAME"])
		tt:AddLine(" ")
		tt:AddLine(hint, 0.2, 1, 0.2, 1)
		tt:AddLine(" ")
		tt:AddLine(hint2, 0.2, 1, 0.2, 1)
	end

	local f = CreateFrame("Frame")
	f:SetScript("OnEvent", function()
		PremoPower.icon = LibStub("LibDBIcon-1.0", true)
		if not PremoPowerLDBIconDB then PremoPowerLDBIconDB = {} end
		PremoPower.icon:Register("PremoPower", plugin, PremoPowerLDBIconDB)
	end)
	f:RegisterEvent("PLAYER_LOGIN")
end

-- ---------------------------------------------
-- Create the default settings for this addon 
-- ---------------------------------------------
local defaults = { 
    profile =  {
		frame = {
			pos = {
				x = 0,
				y = 0,
				scale = 1,
				a = "TOPLEFT",
				r = UIParent,
				rp = "CENTER",
			},
			color = {
				r = 1,
				g = 1,
				b = 1,
				a = 1,
			},
		},
		minimap = {
			hide = false,
		},
	},
}

-- LFX tables
local lfxt = {
	{
		root = true, -- creates the frame but doesn't draw any backdrop
		ftype = "Frame",
		name = "PP_BG",
		childbd = {
			bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
			edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
			tile = true, tileSize = 4, edgeSize = 4,
			insets = { left = 1, right = 1, top = 1, bottom = 1 },
		},
		rgba = {r = 0, g = 0, b = 0, a = 0.7},
		rgbashaded = {r = 0, g = 0, b = 0, a = 0.7},
		childrgba = {r = 246, g = 227, b = 19, a = 0.7},
		childrgbashaded = {r = 246, g = 227, b = 19, a = 0.3},
		childbrgba = {r = 0, g = 0, b = 0, a = 1},
		dim = {w = 200, h = 27, s = 1},
		childdim = {w = 25, h = 25, s = 1},
		pos = {x = 0, y = 0, a = "TOPLEFT", r = UIParent, rp = "CENTER"},
		brgba = {r = 255, g = 255, b = 255, a = 0.7},
		show = true,
		movable = {
			enabled = true, 
			drag = {"RightButton"},
		},
		scripts = {
			{
				event = "OnDragStart", 
				func = function( self )
					self:StartMoving()
				end,
			},
			{
				event = "OnDragStop", 
				func = function( self )
					self:StopMovingOrSizing()
					PremoPower.db.profile.frame.pos.a, PremoPower.db.profile.frame.pos.r, PremoPower.db.profile.frame.pos.rp, PremoPower.db.profile.frame.pos.x, PremoPower.db.profile.frame.pos.y = self:GetPoint(1)
				end,
			},
		},
	},
	{
		ftype = "Frame",
		name = "PP_Frame1",
		text = {
			str = "1",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 1, a = "BOTTOMLEFT", r = "PP_BG", rp = "BOTTOMLEFT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame2",
		text = {
			str = "2",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame1", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame3",
		text = {
			str = "3",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame2", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame4",
		text = {
			str = "4",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame3", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame5",
		text = {
			str = "5",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame4", rp = "BOTTOMRIGHT"},
		show = true,
	},
	{
		ftype = "Frame",
		name = "PP_Frame6",
		text = {
			str = "6",
			color = {
				r = 255,
				g = 255,
				b = 255,
				a = 1,
			},
			fs = "NumberFont_Shadow_Small",
			halign = "CENTER",
			valign = "CENTER",
		},
		parentrgba = true, parentbrgba = true, parentbd = true,	parentdim = true, parentname = "PP_BG",
		pos = {x = 0, y = 0, a = "BOTTOMLEFT", r = "PP_Frame5", rp = "BOTTOMRIGHT"},
		show = true,
	},
}

--[[=============================================================
						OnInitialize
    =============================================================]]
function PremoPower:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PremoPowerDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("PremoPowerDB", self.GenerateOptions)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PremoPower", self.slashcmd, "ppower")
	
	local ACD3 = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.PremoPower = ACD3:AddToBlizOptions("PremoPowerDB", L["ADDON_NAME"], nil)
	self:RegisterModuleOptions("PPSlashCmd", self.slashcmd, L["ADDON_SLASHCMD"])
	self:RegisterModuleOptions("Profiles", function() return LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db) end, L["Profiles"])
	self.optionsFrames.About = ACD3:AddToBlizOptions("PremoPowerDB", L["OPT_ABOUT"], L["ADDON_NAME"], "About")

	LFX:makeFrames(lfxt)
	local pos = self.db.profile.frame.pos
	LFX:setPosition(pos.a, pos.r, pos.rp, pos.x, pos.y)

	self:Print(L["ADDON_NAME"] .. " " .. self.gVersion .. " " .. L["ADDON_LOADED"])
	
	self:CreateMinimapIcon()
	
	self.OnInitialize = nil
end

function PremoPower:RegisterModuleOptions(name, optionTbl, displayName)
	if moduleOptions then
		moduleOptions[name] = optionTbl
	else
		self.options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
	end
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PremoPowerDB", displayName, L["ADDON_NAME"], name)
end

function PremoPower.GenerateOptions()
	if not PremoPower.options then
		PremoPower.GenerateOptionsInternal()
		PremoPower.GenerateOptionsInternal = nil
		moduleOptions = nil
	end
	return PremoPower.options
end

-- Option table for the slash command only
PremoPower.slashcmd = {
	type = "group",
	name = L["ADDON_SLASHCMD"],
	order = -3,
	func = function() PremoPower:ShowConfig() end,
	args = {
		intro = {
			order = 1,
			type = "description",
			name = L["ADDON_SLASHCMD_DESC"],
			cmdHidden = true,
		},
		showicon = {
			order = 6,
			type = "execute",
			name = L["CMD_MINIMAP_SHOW"],
			desc = L["CMD_MINIMAP_SHOW_DESC"],
			func = function()
				PremoPower.db.profile.minimap.hide = false
				PremoPower.icon:Show("PremoPower")
			end,
		},
		hideicon = {
			order = 7,
			type = "execute",
			name = L["CMD_MINIMAP_HIDE"],
			desc = L["CMD_MINIMAP_HIDE_DESC"],
			func = function()
				PremoPower.db.profile.minimap.hide = true 
				PremoPower.icon:Hide("PremoPower")
			end,
		},
		hideicon = {
			order = 8,
			type = "execute",
			name = L["CMD_LOCK"],
			desc = L["CMD_LOCK_DESC"],
			func = function()
				PremoPower.db.profile.minimap.hide = true 
				PremoPower.icon:Hide("PremoPower")
			end,
		},
		config = {
			type = "execute",
			name = L["CMD_CONFIG"],
			desc = L["CMD_CONFIG_DESC"],
			func = function() PremoPower:ShowConfig() end,
			guiHidden = true,
		},
	},
}

function PremoPower.GenerateOptionsInternal()
	-- Create the options variable to use in the config screen --
	PremoPower.options = {
		name = L["OPT_PREMOPOWER_OPTIONS"],
		handler = PremoPower,
		type = "group",
		args = {
			PremoPower = {
				name = L["ADDON_NAME"],
				desc = L["OPT_PP_DESC"],
				type = "group",
				order = 1,
				args = {
					hdrpp = {
						name = L["HDR_DIM"],
						type = "header",
						order = 1,
					},
				},
			},
			About = {
				name = L["OPT_ABOUT"],
				desc = "",
				type = "group",
				order = 3,
				args = {
					desctitle = {
						name = LTC:setColor( L["ADDON_NAME"] .. " " .. PremoPower.gVersion .. " - " .. PremoPower.gCurReleaseDate, LTC_GREEN ),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 1,
					},
					descauthor = {
						name = LTC:setColor(L["ADDON_AUTHOR"] .. " " .. "Azmaedus", LTC_BLUE),
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 2,
					},
					descslashhdr = {
						name = "\r" .. L["ABOUT_SLASHCMD_TITLE"],
						type = "description",
						width = "full",
						fontSize = "medium",
						order = 3,
					},
					descslashplay = {
						name = L["ABOUT_SLASHCMD_CONFIG"],
						type = "description",
						width = "full",
						fontSize = "small",
						order = 4,
					},
				},
			},
		},
	}

	for k, v in pairs(moduleOptions) do
		PremoPower.options.args[k] = (type(v) == "function") and v() or v
	end

	-- Add ordering data to the option table generated by AceDBOptions-3.0
	PremoPower.options.args.Profiles.order = -2
end

function PremoPower:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.Profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.PremoPower)
end

--[[=============================================================
						OnEnable
    =============================================================]]
function PremoPower:OnEnable()
	self:RegisterEvent("UNIT_COMBO_POINTS", "OnUpdateFrames")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnUpdateFrames")
	self:RegisterEvent("UNIT_POWER", "OnUpdateFrames")
	self:RegisterEvent("UNIT_AURA", "OnUpdateFrames")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnUpdateFrames")
	
	PremoPower:OnUpdateFrames()
end

--[[=============================================================
						OnDisable
    =============================================================]]
function PremoPower:OnDisable()
	self:UnregisterEvent("UNIT_COMBO_POINTS")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	self:UnregisterEvent("UNIT_POWER")
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
end

-- -------------------------------------------
-- Events
-- -------------------------------------------
function PremoPower:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
	self.db.profile.frame = db.frame

	LFX:makeFrames(lfxt)
	local pos = self.db.profile.frame.pos
	LFX:setPosition(pos.a, pos.r, pos.rp, pos.x, pos.y)
end
	
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

function PremoPower:OnUpdateFrames()
	if PremoPower.gPlayerInfo.class == "ROGUE" then
		self:updateFrames( MAX_ROGUE_BLIP, GetComboPoints("player" ,"target"), 1 ) -- number of frames to remove from calculation
	elseif PremoPower.gPlayerInfo.class == "MONK" then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_CHI), UnitPower("player", SPELL_POWER_CHI), 1 ) -- number of frames to remove from calculation
	elseif PremoPower.gPlayerInfo.class == "PALADIN" then
		self:updateFrames( UnitPowerMax("player", SPELL_POWER_HOLY_POWER), UnitPower("player", SPELL_POWER_HOLY_POWER), 1 ) -- number of frames to remove from calculation
	elseif PremoPower.gPlayerInfo.class == "PRIEST" then
		if PremoPower.gPlayerInfo.spec == SPEC_PRIEST_DISC then -- disc
			self:updateFrames( UnitPowerMax("player", SPELL_POWER_SHADOW_ORBS), UnitPower( "player", SPELL_POWER_SHADOW_ORBS ), 1 ) -- number of frames to remove from calculation
		elseif PremoPower.gPlayerInfo.spec == SPEC_PRIEST_SHADOW then -- shadow
			self:updateFrames( MAX_PRIEST_DISC_BLIP, getEvangelismStacks(), 1 ) -- number of frames to remove from calculation
		end
	end
end

function PremoPower:getEvangelismStacks()
	local _, _, _, count = UnitAura( "player", "Evangelism" ) --81661
	if count == nil then
		return 0
	else
		return count
	end
end

-- Function to update frames in regards to # of frames/state required
-- total: maximum frames to display
-- [cur]: shadow states for frames after cur(index)
-- [minus]: used as a balance if we do not want to calculate cur with a root
function PremoPower:updateFrames(total, cur, minus)
	total = total + minus
	cur = cur + minus
	for k, v in ipairs(LFX.tframes) do
		if not v.root then
			local rgba = {}
			local rgbashaded = {}
			
			-- This is somewhat ugly, I can do better
			-- parent info or not
			if v.parentrgba then
				local id = LFX:findFrame(v.parentname)
				if id then
					rgba = LFX.tframes[id].childrgba
					rgbashaded = LFX.tframes[id].childrgbashaded
				else
					rgba = LFX.tframes[id].rgba
					rgbashaded = LFX.tframes[id].rgbashaded
				end
			else
				rgba = LFX.tframes[i].rgba
				rgbashaded = LFX.tframes[i].rgbashaded
			end

			-- set backdrop color
			if k <= cur then
				v:Show()
				v:SetBackdropColor(rgba.r/255, rgba.g/255, rgba.b/255, rgba.a)
			elseif k <= total then
				v:Show()
				v:SetBackdropColor(rgbashaded.r/255, rgbashaded.g/255, rgbashaded.b/255, rgbashaded.a)
			elseif k > total then
				v:Hide()
			end
		end
	end
end
